<!-- documentation/library_list - Automated list of all code and documentation files with brief descriptions. -->

# Share File Systems - Code Library List
This is a dynamically compiled list of supporting code files that comprise this application with a brief description of each file.

* Directory *[../documentation](../documentation)*
   - **[code_style.md](code_style.md)**                                                                                                                 - Describes rules for code conformance.
   - **[code_style_explained.md](code_style_explained.md)**                                                                                             - Describes rules for code conformance.
   - **[commands.md](commands.md)**                                                                                                                     - This documentation describes the various supported terminal commands and is automatically generated from `lib/terminal/utilities/commands_documentation.ts`.
   - **[credits.md](credits.md)**                                                                                                                       - A list of external source material not originally created within this project.
   - **[electron_tauri.md](electron_tauri.md)**                                                                                                         - Documentation for launching the application as a desktop application.
   - **[installation.md](installation.md)**                                                                                                             - Describes the installation process and the various files and commands involved.
   - **[library_list.md](library_list.md)**                                                                                                             - Automated list of all code and documentation files with brief descriptions.
   - **[linuxVM.md](linuxVM.md)**                                                                                                                       - Notes about configuring Linux virtual machines in support of project development.
   - **[modal.md](modal.md)**                                                                                                                           - Notes about modals and the graphic user interface that displays in the browser.
   - **[screenshots.md](screenshots.md)**                                                                                                               - A list of screenshots demonstrating what the application looks like.
   - **[security_model.md](security_model.md)**                                                                                                         - Describes the application's security model.
   - **[test_browser.md](test_browser.md)**                                                                                                             - How this application achieves test automation.
* Directory *[../lib](../lib)*
   - **[../lib/electron.ts](../lib/electron.ts)**                                                                                                       - API endpoint to electron desktop application wrapper.
   - **[../lib/tauri.ts](../lib/tauri.ts)**                                                                                                             - API endpoint to Tauri desktop application wrapper.
* Directory *[../lib/browser](../lib/browser)*
   - **[../lib/browser/browser.ts](../lib/browser/browser.ts)**                                                                                         - A list of declared variables globally available to the browser instance of the application.
   - **[../lib/browser/index.ts](../lib/browser/index.ts)**                                                                                             - The base JavaScript code that initiates the application in the browser.
* Directory *[../lib/browser/content](../lib/browser/content)*
   - **[../lib/browser/content/agent_management.ts](../lib/browser/content/agent_management.ts)**                                                       - Provide control of agent data: delete, invite, and edit.
   - **[../lib/browser/content/configuration.ts](../lib/browser/content/configuration.ts)**                                                             - A collection of utilities and event handlers associated with processing the application state and system configuration.
   - **[../lib/browser/content/context.ts](../lib/browser/content/context.ts)**                                                                         - A collection of event handlers associated with the right click context menu.
   - **[../lib/browser/content/file_browser.ts](../lib/browser/content/file_browser.ts)**                                                               - A collection of utilities for handling file system related tasks in the browser.
   - **[../lib/browser/content/global_events.ts](../lib/browser/content/global_events.ts)**                                                             - Events not associated with specific content or a modal type.
   - **[../lib/browser/content/media.ts](../lib/browser/content/media.ts)**                                                                             - A library for executing audio/video calls.
   - **[../lib/browser/content/message.ts](../lib/browser/content/message.ts)**                                                                         - A library for executing the text messaging application.
   - **[../lib/browser/content/share.ts](../lib/browser/content/share.ts)**                                                                             - The utilities that manage and coordinate changes to user share data.
   - **[../lib/browser/content/tutorial.ts](../lib/browser/content/tutorial.ts)**                                                                       - An interactive tutorial explaining the application.
* Directory *[../lib/browser/utilities](../lib/browser/utilities)*
   - **[../lib/browser/utilities/agent_hash.ts](../lib/browser/utilities/agent_hash.ts)**                                                               - Generates a local user identity.
   - **[../lib/browser/utilities/agent_status.ts](../lib/browser/utilities/agent_status.ts)**                                                           - Receive and process agent activity status notifications from the network.
   - **[../lib/browser/utilities/audio.ts](../lib/browser/utilities/audio.ts)**                                                                         - A storage of audio samples encoded in Base64 with respective metadata.
   - **[../lib/browser/utilities/dom.ts](../lib/browser/utilities/dom.ts)**                                                                             - Extensions to the DOM to provide navigational functionality not present from the standard methods
   - **[../lib/browser/utilities/modal.ts](../lib/browser/utilities/modal.ts)**                                                                         - A collection of utilities for generating and manipulating modals/windows in the browser.
   - **[../lib/browser/utilities/network.ts](../lib/browser/utilities/network.ts)**                                                                     - The methods that execute data requests to the local terminal instance of the application.
   - **[../lib/browser/utilities/remote.ts](../lib/browser/utilities/remote.ts)**                                                                       - A collection of instructions to allow event execution from outside the browser, like a remote control.
   - **[../lib/browser/utilities/util.ts](../lib/browser/utilities/util.ts)**                                                                           - Miscellaneous tools for the browser environment.
   - **[../lib/browser/utilities/webSocket.ts](../lib/browser/utilities/webSocket.ts)**                                                                 - Handles web socket events and associated errors. This where most communications from outside the browser are processed.
* Directory *[../lib/common](../lib/common)*
   - **[../lib/common/common.ts](../lib/common/common.ts)**                                                                                             - A collection of tools available to any environment.
   - **[../lib/common/disallowed.ts](../lib/common/disallowed.ts)**                                                                                     - Reassignments from default conventions that either dated or most frequently misused.
* Directory *[../lib/terminal/commands/interface](../lib/terminal/commands/interface)*
   - **[../lib/terminal/commands/interface/agent_data.ts](../lib/terminal/commands/interface/agent_data.ts)**                                           - Shell interface for agent_data which forms a report of agent data points.
   - **[../lib/terminal/commands/interface/agent_online.ts](../lib/terminal/commands/interface/agent_online.ts)**                                       - Shell interface for agent_online which is a connectivity tester to remote agents.
   - **[../lib/terminal/commands/interface/base64.ts](../lib/terminal/commands/interface/base64.ts)**                                                   - Shell interface for base64, which provides base64 encoding/decoding.
   - **[../lib/terminal/commands/interface/build.ts](../lib/terminal/commands/interface/build.ts)**                                                     - Shell interface to the build tool.
   - **[../lib/terminal/commands/interface/certificate.ts](../lib/terminal/commands/interface/certificate.ts)**                                         - Shell interface for creating certificates.
   - **[../lib/terminal/commands/interface/commands.ts](../lib/terminal/commands/interface/commands.ts)**                                               - Shell interface for generating dynamic command documentation.
   - **[../lib/terminal/commands/interface/copy.ts](../lib/terminal/commands/interface/copy.ts)**                                                       - Shell interface for the file copy command.
   - **[../lib/terminal/commands/interface/directory.ts](../lib/terminal/commands/interface/directory.ts)**                                             - Shell interface to the directory library that walks the file system and returns a data structure.
   - **[../lib/terminal/commands/interface/firewall.ts](../lib/terminal/commands/interface/firewall.ts)**                                               - Shell interface to the firewall library, which opens the firewall for this application.
   - **[../lib/terminal/commands/interface/get.ts](../lib/terminal/commands/interface/get.ts)**                                                         - Shell interface for http get command.
   - **[../lib/terminal/commands/interface/hash.ts](../lib/terminal/commands/interface/hash.ts)**                                                       - Shell interface to library hash, which generates a hash string.
   - **[../lib/terminal/commands/interface/lint.ts](../lib/terminal/commands/interface/lint.ts)**                                                       - Shell interface for executing TypeScript lint as configured by this application.
   - **[../lib/terminal/commands/interface/mkdir.ts](../lib/terminal/commands/interface/mkdir.ts)**                                                     - Shell interface to utility mkdir for creating directory structures recursively.
   - **[../lib/terminal/commands/interface/remove.ts](../lib/terminal/commands/interface/remove.ts)**                                                   - Shell interface for removing file system artifacts.
   - **[../lib/terminal/commands/interface/service.ts](../lib/terminal/commands/interface/service.ts)**                                                 - Shell interface for running the application's network services, the applications default command.
   - **[../lib/terminal/commands/interface/test.ts](../lib/terminal/commands/interface/test.ts)**                                                       - Shell interface wrapping all test automation utilities.
   - **[../lib/terminal/commands/interface/test_browser.ts](../lib/terminal/commands/interface/test_browser.ts)**                                       - Shell interface for tests to be sent to the browser to impose changes to the DOM and test the result.
   - **[../lib/terminal/commands/interface/test_service.ts](../lib/terminal/commands/interface/test_service.ts)**                                       - Shell interface for the service tests, which test the various services used by the application.
   - **[../lib/terminal/commands/interface/test_simulation.ts](../lib/terminal/commands/interface/test_simulation.ts)**                                 - Shell interface for running simulation tests of supported terminal commands.
   - **[../lib/terminal/commands/interface/typescript.ts](../lib/terminal/commands/interface/typescript.ts)**                                           - Interface to execute TypeScript type evaluation.
   - **[../lib/terminal/commands/interface/version.ts](../lib/terminal/commands/interface/version.ts)**                                                 - Shell interface for expressing the application's version.
   - **[../lib/terminal/commands/interface/websocket.ts](../lib/terminal/commands/interface/websocket.ts)**                                             - Shell interface to start a websocket server from the terminal.
* Directory *[../lib/terminal/commands/library](../lib/terminal/commands/library)*
   - **[../lib/terminal/commands/library/agent_data.ts](../lib/terminal/commands/library/agent_data.ts)**                                               - Forms a report of agent data.
   - **[../lib/terminal/commands/library/agent_online.ts](../lib/terminal/commands/library/agent_online.ts)**                                           - A connectivity tester to shared remote agents.
   - **[../lib/terminal/commands/library/base64.ts](../lib/terminal/commands/library/base64.ts)**                                                       - A utility for performing base64 encoding/decoding.
   - **[../lib/terminal/commands/library/build.ts](../lib/terminal/commands/library/build.ts)**                                                         - The library that executes the build and test tasks.
   - **[../lib/terminal/commands/library/certificate.ts](../lib/terminal/commands/library/certificate.ts)**                                             - A command driven utility for creating an HTTPS certificate.
   - **[../lib/terminal/commands/library/commands.ts](../lib/terminal/commands/library/commands.ts)**                                                   - Utility to list available commands and their respective documentation.
   - **[../lib/terminal/commands/library/copy.ts](../lib/terminal/commands/library/copy.ts)**                                                           - A command driven utility to perform bit by bit file artifact copy.
   - **[../lib/terminal/commands/library/directory.ts](../lib/terminal/commands/library/directory.ts)**                                                 - A utility to walk the file system and return a data structure.
   - **[../lib/terminal/commands/library/firewall.ts](../lib/terminal/commands/library/firewall.ts)**                                                   - A utility to open firewall settings for this application.
   - **[../lib/terminal/commands/library/get.ts](../lib/terminal/commands/library/get.ts)**                                                             - A command driven utility to fetch resources from across the internet via HTTP method GET.
   - **[../lib/terminal/commands/library/hash.ts](../lib/terminal/commands/library/hash.ts)**                                                           - A utility to generate hash sequences on strings or file system artifacts.
   - **[../lib/terminal/commands/library/lint.ts](../lib/terminal/commands/library/lint.ts)**                                                           - Executes a TypeScript configured form of ESLint from the application's dev dependencies.
   - **[../lib/terminal/commands/library/mkdir.ts](../lib/terminal/commands/library/mkdir.ts)**                                                         - A utility for recursively creating directories in the file system.
   - **[../lib/terminal/commands/library/remove.ts](../lib/terminal/commands/library/remove.ts)**                                                       - A command driven utility to recursively remove file system artifacts.
   - **[../lib/terminal/commands/library/typescript.ts](../lib/terminal/commands/library/typescript.ts)**                                               - Executes TypeScript type checks.
   - **[../lib/terminal/commands/library/update.ts](../lib/terminal/commands/library/update.ts)**                                                       - A command to update the application from git and then run the build.
* Directory *[../lib/terminal/server](../lib/terminal/server)*
   - **[../lib/terminal/server/osNotification.ts](../lib/terminal/server/osNotification.ts)**                                                           - This library sends user messaging notifications to the operating system.
   - **[../lib/terminal/server/readCerts.ts](../lib/terminal/server/readCerts.ts)**                                                                     - Reads certificates for secure transmission protocol support
* Directory *[../lib/terminal/server/services](../lib/terminal/server/services)*
   - **[../lib/terminal/server/services/agent_hash.ts](../lib/terminal/server/services/agent_hash.ts)**                                                 - A library for creating a new user/device identification.
   - **[../lib/terminal/server/services/agent_management.ts](../lib/terminal/server/services/agent_management.ts)**                                     - Add, delete, and modify agent data.
   - **[../lib/terminal/server/services/agent_online.ts](../lib/terminal/server/services/agent_online.ts)**                                             - Determines if a remote agent is online and if so gathers their IP addresses and listening port numbers.
   - **[../lib/terminal/server/services/agent_status.ts](../lib/terminal/server/services/agent_status.ts)**                                             - Publishes activity status of agents.
   - **[../lib/terminal/server/services/browserLog.ts](../lib/terminal/server/services/browserLog.ts)**                                                 - This handy utility writes log output to the terminal from the browser's console.log for more direct log visibility.
   - **[../lib/terminal/server/services/deviceMask.ts](../lib/terminal/server/services/deviceMask.ts)**                                                 - A library to mask/unmask masked device identities communicated between different users.
   - **[../lib/terminal/server/services/fileCopy.ts](../lib/terminal/server/services/fileCopy.ts)**                                                     - A library that stores instructions for copy and cut of file system artifacts.
   - **[../lib/terminal/server/services/fileExecute.ts](../lib/terminal/server/services/fileExecute.ts)**                                               - A common file execution library used by both fileCopy and fileSystem.
   - **[../lib/terminal/server/services/fileSystem.ts](../lib/terminal/server/services/fileSystem.ts)**                                                 - Manages various file system services.
   - **[../lib/terminal/server/services/hashShare.ts](../lib/terminal/server/services/hashShare.ts)**                                                   - Creates a unique identifier for a new share object.
   - **[../lib/terminal/server/services/invite.ts](../lib/terminal/server/services/invite.ts)**                                                         - Manages the order of invitation related processes for traffic across the internet.
   - **[../lib/terminal/server/services/message.ts](../lib/terminal/server/services/message.ts)**                                                       - Process and send text messages.
   - **[../lib/terminal/server/services/settings.ts](../lib/terminal/server/services/settings.ts)**                                                     - A library for writing data to settings.
* Directory *[../lib/terminal/server/transmission](../lib/terminal/server/transmission)*
   - **[../lib/terminal/server/transmission/ipResolve.ts](../lib/terminal/server/transmission/ipResolve.ts)**                                           - Tests connectivity to remote agents from among their known IP addresses.
   - **[../lib/terminal/server/transmission/methodGET.ts](../lib/terminal/server/transmission/methodGET.ts)**                                           - The library for handling all traffic related to HTTP requests with method GET.
   - **[../lib/terminal/server/transmission/receiver.ts](../lib/terminal/server/transmission/receiver.ts)**                                             - The library for handling all traffic related to HTTP requests with method POST.
   - **[../lib/terminal/server/transmission/responder.ts](../lib/terminal/server/transmission/responder.ts)**                                           - Send network output, whether an http response or websocket.
   - **[../lib/terminal/server/transmission/sender.ts](../lib/terminal/server/transmission/sender.ts)**                                                 - Abstracts away the communication channel from the message.
   - **[../lib/terminal/server/transmission/transmit_http.ts](../lib/terminal/server/transmission/transmit_http.ts)**                                   - This library launches the HTTP service and all supporting service utilities.
   - **[../lib/terminal/server/transmission/transmit_logger.ts](../lib/terminal/server/transmission/transmit_logger.ts)**                               - Logs transmission data to the console when in verbose mode.
   - **[../lib/terminal/server/transmission/transmit_ws.ts](../lib/terminal/server/transmission/transmit_ws.ts)**                                       - A command utility for creating a websocket server or client.
* Directory *[../lib/terminal/test/application](../lib/terminal/test/application)*
   - **[../lib/terminal/test/application/browser.ts](../lib/terminal/test/application/browser.ts)**                                                     - The functions necessary to run browser test automation.
   - **[../lib/terminal/test/application/evaluation.ts](../lib/terminal/test/application/evaluation.ts)**                                               - Evaluate a given test item and report appropriate failure messaging.
   - **[../lib/terminal/test/application/runner.ts](../lib/terminal/test/application/runner.ts)**                                                       - A test runner that loops through test items in serial, executes those test items, and passes the result message to the evaluation library.
   - **[../lib/terminal/test/application/service.ts](../lib/terminal/test/application/service.ts)**                                                     - A list of service test related utilities.
   - **[../lib/terminal/test/application/simulation.ts](../lib/terminal/test/application/simulation.ts)**                                               - A list of command related tests for running shell simulations against the supported commands.
   - **[../lib/terminal/test/application/testComplete.ts](../lib/terminal/test/application/testComplete.ts)**                                           - Final messaging for a completed test type.
* Directory *[../lib/terminal/test/application/browserUtilities](../lib/terminal/test/application/browserUtilities)*
   - **[../lib/terminal/test/application/browserUtilities/file_path_decode.ts](../lib/terminal/test/application/browserUtilities/file_path_decode.ts)** - Transforms a custom encoded file path into a local operation system specific file path.
   - **[../lib/terminal/test/application/browserUtilities/file_path_encode.ts](../lib/terminal/test/application/browserUtilities/file_path_encode.ts)** - Creates an encoding around file system addresses so that the test code can ensure the paths are properly formed per operating system.
   - **[../lib/terminal/test/application/browserUtilities/inviteAccept.ts](../lib/terminal/test/application/browserUtilities/inviteAccept.ts)**         - A test generator for accepting an invitation.
   - **[../lib/terminal/test/application/browserUtilities/inviteConfirm.ts](../lib/terminal/test/application/browserUtilities/inviteConfirm.ts)**       - A test generator for accepting an invitation.
   - **[../lib/terminal/test/application/browserUtilities/inviteModal.ts](../lib/terminal/test/application/browserUtilities/inviteModal.ts)**           - A test generator for spawning the invitation modal.
   - **[../lib/terminal/test/application/browserUtilities/inviteSend.ts](../lib/terminal/test/application/browserUtilities/inviteSend.ts)**             - A test generator for sending an invitation.
   - **[../lib/terminal/test/application/browserUtilities/login.ts](../lib/terminal/test/application/browserUtilities/login.ts)**                       - A test generator to login a fresh device.
   - **[../lib/terminal/test/application/browserUtilities/machines.ts](../lib/terminal/test/application/browserUtilities/machines.ts)**                 - An object expressing a map of the various remote machines.
   - **[../lib/terminal/test/application/browserUtilities/mainMenu.ts](../lib/terminal/test/application/browserUtilities/mainMenu.ts)**                 - A convenience function that opens the main menu while in browser tests.
   - **[../lib/terminal/test/application/browserUtilities/modalAddress.ts](../lib/terminal/test/application/browserUtilities/modalAddress.ts)**         - A convenience function that tests a file navigation modal to go to the project's location for browser tests.
   - **[../lib/terminal/test/application/browserUtilities/moveToSandbox.ts](../lib/terminal/test/application/browserUtilities/moveToSandbox.ts)**       - Generates a browser test to move a file navigate modal to the project's internal test location.
   - **[../lib/terminal/test/application/browserUtilities/newDirectory.ts](../lib/terminal/test/application/browserUtilities/newDirectory.ts)**         - Generates a browser test to create new directories in the file system.
   - **[../lib/terminal/test/application/browserUtilities/showContextMenu.ts](../lib/terminal/test/application/browserUtilities/showContextMenu.ts)**   - A convenience function that launches the modal context menu in browser tests.
* Directory *[../lib/terminal/test/samples](../lib/terminal/test/samples)*
   - **[../lib/terminal/test/samples/browser_device.ts](../lib/terminal/test/samples/browser_device.ts)**                                               - A list of tests that execute in the web browser and require multiple computers.
   - **[../lib/terminal/test/samples/browser_self.ts](../lib/terminal/test/samples/browser_self.ts)**                                                   - A list of tests that execute in the web browser only on this computer.
   - **[../lib/terminal/test/samples/browser_user.ts](../lib/terminal/test/samples/browser_user.ts)**                                                   - A list of tests that execute in the web browser and require multiple computers.
   - **[../lib/terminal/test/samples/service.ts](../lib/terminal/test/samples/service.ts)**                                                             - A list of service tests.
   - **[../lib/terminal/test/samples/simulation.ts](../lib/terminal/test/samples/simulation.ts)**                                                       - A list of command related tests for running shell simulations against the supported commands.
* Directory *[../lib/terminal/utilities](../lib/terminal/utilities)*
   - **[../lib/terminal/utilities/commandList.ts](../lib/terminal/utilities/commandList.ts)**                                                           - Groups all supported command functions into an object for single point of reference.
   - **[../lib/terminal/utilities/commandName.ts](../lib/terminal/utilities/commandName.ts)**                                                           - A library for visually presenting command documentation to the terminal.
   - **[../lib/terminal/utilities/commands_documentation.ts](../lib/terminal/utilities/commands_documentation.ts)**                                     - A data structure defining command documentation with usage examples.
   - **[../lib/terminal/utilities/entry.ts](../lib/terminal/utilities/entry.ts)**                                                                       - The entry point to the application.
   - **[../lib/terminal/utilities/error.ts](../lib/terminal/utilities/error.ts)**                                                                       - A utility for processing and logging errors from the terminal application.
   - **[../lib/terminal/utilities/getAddress.ts](../lib/terminal/utilities/getAddress.ts)**                                                             - Extracts IP addresses from a provided socket.
   - **[../lib/terminal/utilities/humanTime.ts](../lib/terminal/utilities/humanTime.ts)**                                                               - A utility to generate human readable time sequences.
   - **[../lib/terminal/utilities/ipList.ts](../lib/terminal/utilities/ipList.ts)**                                                                     - Returns a list of ip addresses for a specified agent.
   - **[../lib/terminal/utilities/lists.ts](../lib/terminal/utilities/lists.ts)**                                                                       - A utility for visually presenting lists of data to the terminal's console.
   - **[../lib/terminal/utilities/log.ts](../lib/terminal/utilities/log.ts)**                                                                           - A log utility for displaying multiple lines of text to the terminal.
   - **[../lib/terminal/utilities/readStorage.ts](../lib/terminal/utilities/readStorage.ts)**                                                           - Reads all the settings files and returns a data structure to a callback
   - **[../lib/terminal/utilities/rename.ts](../lib/terminal/utilities/rename.ts)**                                                                     - Before creating new file system artifacts this library determines if the artifact is already present and if so changes the name of the new artifacts to prevent overwrite.
   - **[../lib/terminal/utilities/terminal.ts](../lib/terminal/utilities/terminal.ts)**                                                                 - Execute the application entry point from the terminal.
   - **[../lib/terminal/utilities/time.ts](../lib/terminal/utilities/time.ts)**                                                                         - Generates a timestamp in format: "[HH:mm:ss:mil] message".
   - **[../lib/terminal/utilities/vars.ts](../lib/terminal/utilities/vars.ts)**                                                                         - Globally available variables for the terminal utility.
   - **[../lib/terminal/utilities/wrapIt.ts](../lib/terminal/utilities/wrapIt.ts)**                                                                     - A tool to perform word wrap when printing text to the shell.
   - **[../lib/terminal/utilities/writeStream.ts](../lib/terminal/utilities/writeStream.ts)**                                                           - A utility to pipe from a read stream to a write stream.
* Directory *[../lib/typescript](../lib/typescript)*
   - **[../lib/typescript/browser.d.ts](../lib/typescript/browser.d.ts)**                                                                               - TypeScript interfaces used by browser specific libraries.
   - **[../lib/typescript/config.d.ts](../lib/typescript/config.d.ts)**                                                                                 - TypeScript interfaces defining method configurations.
   - **[../lib/typescript/environment.d.ts](../lib/typescript/environment.d.ts)**                                                                       - TypeScript interfaces that define environmental objects.
   - **[../lib/typescript/fileService.d.ts](../lib/typescript/fileService.d.ts)**                                                                       - TypeScript interfaces used by the file services.
   - **[../lib/typescript/global.d.ts](../lib/typescript/global.d.ts)**                                                                                 - TypeScript interfaces used in many unrelated areas of the application.
   - **[../lib/typescript/modules_browser.d.ts](../lib/typescript/modules_browser.d.ts)**                                                               - TypeScript interfaces that define master library modules used in the browser.
   - **[../lib/typescript/modules_terminal.d.ts](../lib/typescript/modules_terminal.d.ts)**                                                             - TypeScript interfaces that define master library modules used in the terminal.
   - **[../lib/typescript/services.d.ts](../lib/typescript/services.d.ts)**                                                                             - Stores definitions of the various service data objects, such as those that comprise the socketData transfer type.
   - **[../lib/typescript/terminal.d.ts](../lib/typescript/terminal.d.ts)**                                                                             - TypeScript interfaces used by terminal specific libraries.
   - **[../lib/typescript/test.d.ts](../lib/typescript/test.d.ts)**                                                                                     - TypeScript interfaces used test automation.
   - **[../lib/typescript/types.d.ts](../lib/typescript/types.d.ts)**                                                                                   - TypeScript static types.