/* lib/terminal/server/services/message - Process and send text messages. */ import { createReadStream, createWriteStream, readdir } from "fs";
import error from "../../utilities/error.js";
import osNotification from "../osNotification.js";
import sender from "../transmission/sender.js";
import settings from "./settings.js";
import vars from "../../utilities/vars.js";
const message = function terminal_server_services_message(socketData) {
    // broadcasts and offline messaging are exclusive
    // data length greater than 1 only applies to sending or receiving offline messages
    const data = socketData.data, count = 500, broadcast = function terminal_server_services_message_broadcast(agentType) {
        const list = Object.keys(vars.settings[agentType]);
        let agentLength = list.length;
        do {
            agentLength = agentLength - 1;
            if (agentType === "user" || agentType === "device" && list[agentLength] !== vars.settings.hashDevice) {
                data[0].message = `(broadcast) ${data[0].message}`;
                sender.send({
                    data: data,
                    service: "message"
                }, {
                    device: data[0].agentType === "device" ? data[0].agentTo : "",
                    user: data[0].agentType === "device" ? vars.settings.hashUser : data[0].agentTo
                });
            }
            if (agentType === "device") {
                osNotification();
            }
        }while (agentLength > 0)
    }, write = function terminal_server_services_message_write() {
        const save = function terminal_server_services_message_write_save() {
            settings({
                data: {
                    settings: vars.settings.message,
                    type: "message"
                },
                service: "message"
            });
        };
        if (vars.settings.message.length > count) {
            readdir(`${vars.path.project}lib${vars.path.sep}settings${vars.path.sep}message_archive`, function terminal_server_services_message_readdir(erd, files) {
                if (erd === null) {
                    const fileName = function terminal_server_services_message_readdir_fileName() {
                        const test = /message\d+\.json/, numb = function terminal_server_services_message_readdir_fileName_numb(input) {
                            return Number(input.replace("message", "").replace(".json", ""));
                        }, sort = function terminal_server_services_message_readdir_fileName_sort(itemA, itemB) {
                            if (test.test(itemB) === true && test.test(itemA) === false) {
                                return 1;
                            }
                            if (test.test(itemA) === true && test.test(itemB) === false) {
                                return -1;
                            }
                            if (numb(itemA) > numb(itemB)) {
                                return -1;
                            }
                            return 1;
                        };
                        files.sort(sort);
                        if (test.test(files[0]) === true) {
                            return `message${numb(files[0]) + 1}.json`;
                        }
                        return "message0.json";
                    }(), readStream = createReadStream(JSON.stringify(vars.settings.message.slice(0, count))), writeStream = createWriteStream(`${vars.path.project}lib${vars.path.sep}settings${vars.path.sep}message_archive${vars.path.sep + fileName}`);
                    readStream.pipe(writeStream);
                    writeStream.on("finish", function terminal_server_services_message_readdir_writeFinish() {
                        vars.settings.message = vars.settings.message.slice(count);
                        save();
                    });
                    writeStream.on("error", function terminal_server_services_message_readdir_writeError(errMessage) {
                        error([
                            errMessage.toString()
                        ]);
                    });
                } else {
                    error([
                        "Error performing readdir on message_archive",
                        erd.toString()
                    ]);
                }
            });
        } else {
            save();
        }
    };
    if (data[0].agentTo === "device") {
        broadcast("device");
    } else if (data[0].agentTo === "user") {
        broadcast("user");
    } else if (data[0].agentTo === "all") {
        broadcast("device");
        broadcast("user");
    } else if (data[0].agentType === "device" && data[0].agentTo === vars.settings.hashDevice) {
        sender.broadcast({
            data: data,
            service: "message"
        }, "browser");
        osNotification();
    } else if (data[0].agentType === "user" && data[0].agentTo === vars.settings.hashUser) {
        sender.broadcast({
            data: data,
            service: "message"
        }, "browser");
        broadcast("device");
    } else {
        if (vars.settings[data[0].agentType][data[0].agentTo].status === "offline") {
            data.forEach(function terminal_server_services_message_offline(item) {
                item.offline = true;
            });
        } else {
            sender.send({
                data: data,
                service: "message"
            }, {
                device: data[0].agentType === "device" ? data[0].agentTo : "",
                user: data[0].agentType === "device" ? vars.settings.hashUser : data[0].agentTo
            });
        }
    }
    vars.settings.message = vars.settings.message.concat(data);
    write();
};
export default message;
